@@ -57,9 +57,9 @@ class QkdProgram(Program):
 
         for i in range(self._num_bits):
             if create:
-                q = epr_socket.create(1)[0]
+                q = epr_socket.create_keep(1)[0]
             else:
-                q = epr_socket.recv(1)[0]
+                q = epr_socket.recv_keep(1)[0]
             if bases[i] == 1:
                 q.H()
             m = q.measure()
@@ -146,7 +146,6 @@ class AliceProgram(QkdProgram):
     def meta(self) -> ProgramMeta:
         return ProgramMeta(
             name="alice_program",
-            parameters={"num_bits": self._num_bits},
             csockets=[self.PEER],
             epr_sockets=[self.PEER],
             max_qubits=1,
@@ -208,7 +207,6 @@ class BobProgram(QkdProgram):
     def meta(self) -> ProgramMeta:
         return ProgramMeta(
             name="bob_program",
-            parameters={"num_bits": self._num_bits},
             csockets=[self.PEER],
             epr_sockets=[self.PEER],
             max_qubits=1,
