@@ -42,7 +42,11 @@ from squidasm.sim.stack.common import (
     PortListener,
 )
 from squidasm.sim.stack.globals import GlobalSimData
-from squidasm.sim.stack.signals import SIGNAL_HAND_PROC_MSG, SIGNAL_NSTK_PROC_MSG
+from squidasm.sim.stack.signals import (
+    SIGNAL_HAND_PROC_MSG,
+    SIGNAL_MEMORY_FREED,
+    SIGNAL_NSTK_PROC_MSG,
+)
 
 if TYPE_CHECKING:
     from squidasm.sim.stack.qnos import Qnos
@@ -52,6 +56,19 @@ PI_OVER_2 = math.pi / 2
 
 
 class ProcessorComponent(Component):
+    """NetSquid component representing a QNodeOS processor.
+
+    Subcomponent of a QnosComponent.
+
+    Has communications ports with
+     - the netstack component of this QNodeOS
+     - the handler compmonent of this QNodeOS
+
+    This is a static container for processor-related components and ports.
+    Behavior of a QNodeOS processor is modeled in the `Processor` class,
+    which is a subclass of `Protocol`.
+    """
+
     def __init__(self, node: Node) -> None:
         super().__init__(f"{node.name}_processor")
         self._node = node
@@ -84,7 +101,15 @@ class ProcessorComponent(Component):
 
 
 class Processor(ComponentProtocol):
+    """NetSquid protocol representing a QNodeOS processor."""
+
     def __init__(self, comp: ProcessorComponent, qnos: Qnos) -> None:
+        """Processor protocol constructor. Typically created indirectly through
+        constructing a `Qnos` instance.
+
+        :param comp: NetSquid component representing the processor
+        :param qnos: `Qnos` protocol that owns this protocol
+        """
         super().__init__(name=f"{comp.name}_protocol", comp=comp)
         self._comp = comp
         self._qnos = qnos
@@ -98,18 +123,21 @@ class Processor(ComponentProtocol):
             PortListener(self._comp.ports["nstk_in"], SIGNAL_NSTK_PROC_MSG),
         )
 
-        self.add_signal("memory free")  # TODO
+        self.add_signal(SIGNAL_MEMORY_FREED)
 
     @property
     def app_memories(self) -> Dict[int, AppMemory]:
+        """Get a dictionary of app IDs to application memories."""
         return self._qnos.app_memories
 
     @property
     def physical_memory(self) -> PhysicalQuantumMemory:
+        """Get the physical quantum memory object."""
         return self._qnos.physical_memory
 
     @property
     def qdevice(self) -> QuantumProcessor:
+        """Get the NetSquid `QuantumProcessor` object of this node."""
         return self._comp.qdevice
 
     def _send_handler_msg(self, msg: str) -> None:
@@ -128,6 +156,7 @@ class Processor(ComponentProtocol):
         self._listeners["netstack"].buffer.clear()
 
     def run(self) -> Generator[EventExpression, None, None]:
+        """Run this protocol. Automatically called by NetSquid during simulation."""
         while True:
             subroutine = yield from self._receive_handler_msg()
             # assert isinstance(subroutine, Subroutine)
@@ -140,12 +169,13 @@ class Processor(ComponentProtocol):
     def execute_subroutine(
         self, subroutine: Subroutine
     ) -> Generator[EventExpression, None, None]:
+        """Execute a NetQASM subroutine on this processor."""
         app_id = subroutine.app_id
         assert app_id in self.app_memories
         app_mem = self.app_memories[app_id]
         app_mem.set_prog_counter(0)
-        while app_mem.prog_counter < len(subroutine.commands):
-            instr = subroutine.commands[app_mem.prog_counter]
+        while app_mem.prog_counter < len(subroutine.instructions):
+            instr = subroutine.instructions[app_mem.prog_counter]
             self._logger.debug(
                 f"{ns.sim_time()} interpreting instruction {instr} at line {app_mem.prog_counter}"
             )
@@ -256,7 +286,15 @@ class Processor(ComponentProtocol):
         self.app_memories[app_id].set_reg_value(instr.reg, instr.imm.value)
 
     def _interpret_qalloc(self, app_id: int, instr: core.QAllocInstruction) -> None:
-        raise NotImplementedError
+        app_mem = self.app_memories[app_id]
+
+        virt_id = app_mem.get_reg_value(instr.reg)
+        if virt_id is None:
+            raise RuntimeError(f"qubit address in register {instr.reg} is not defined")
+        self._logger.debug(f"Allocating qubit with virtual ID {virt_id}")
+
+        phys_id = self.physical_memory.allocate()
+        app_mem.map_virt_id(virt_id, phys_id)
 
     def _interpret_qfree(self, app_id: int, instr: core.QFreeInstruction) -> None:
         app_mem = self.app_memories[app_id]
@@ -268,7 +306,7 @@ class Processor(ComponentProtocol):
         assert phys_id is not None
         app_mem.unmap_virt_id(virt_id)
         self.physical_memory.free(phys_id)
-        self.send_signal("memory free")  # TODO
+        self.send_signal(SIGNAL_MEMORY_FREED)
         self.qdevice.mem_positions[phys_id].in_use = False
 
     def _interpret_store(self, app_id: int, instr: core.StoreInstruction) -> None:
@@ -454,7 +492,6 @@ class Processor(ComponentProtocol):
             f"{virt_id0} and {virt_id1} (physical IDs: {phys_id0} and {phys_id1})"
         )
         prog = QuantumProgram()
-        # self._logger.warning(f"applying {ns_instr}")
         prog.apply(ns_instr, qubit_indices=[phys_id0, phys_id1], angle=angle)
         yield self.qdevice.execute_program(prog)
 
@@ -464,7 +501,7 @@ class Processor(ComponentProtocol):
         raise NotImplementedError
 
     def _get_rotation_angle_from_operands(self, n: int, d: int) -> float:
-        return float(n * PI / (2 ** d))
+        return float(n * PI / (2**d))
 
     def _interpret_meas(
         self, app_id: int, instr: core.MeasInstruction
@@ -586,16 +623,7 @@ class Processor(ComponentProtocol):
 
 
 class GenericProcessor(Processor):
-    def _interpret_qalloc(self, app_id: int, instr: core.QAllocInstruction) -> None:
-        app_mem = self.app_memories[app_id]
-
-        virt_id = app_mem.get_reg_value(instr.reg)
-        if virt_id is None:
-            raise RuntimeError(f"qubit address in register {instr.reg} is not defined")
-        self._logger.debug(f"Allocating qubit with virtual ID {virt_id}")
-
-        phys_id = self.physical_memory.allocate()
-        app_mem.map_virt_id(virt_id, phys_id)
+    """A `Processor` for nodes with a generic quantum hardware."""
 
     def _interpret_init(
         self, app_id: int, instr: core.InitInstruction
@@ -692,6 +720,8 @@ class GenericProcessor(Processor):
 
 
 class NVProcessor(Processor):
+    """A `Processor` for nodes with a NV hardware."""
+
     def _interpret_qalloc(self, app_id: int, instr: core.QAllocInstruction) -> None:
         app_mem = self.app_memories[app_id]
 
@@ -700,7 +730,11 @@ class NVProcessor(Processor):
             raise RuntimeError(f"qubit address in register {instr.reg} is not defined")
         self._logger.debug(f"Allocating qubit with virtual ID {virt_id}")
 
-        phys_id = self.physical_memory.allocate()
+        # Virtual ID > 0 corresponds to memory qubits
+        if virt_id > 0:
+            phys_id = self.physical_memory.allocate_mem()
+        else:
+            phys_id = self.physical_memory.allocate_comm()
         app_mem.map_virt_id(virt_id, phys_id)
 
     def _interpret_init(
@@ -793,7 +827,7 @@ class NVProcessor(Processor):
                 app_mem.map_virt_id(virt_id, 0)
                 yield from self._move_carbon_to_electron_for_measure(phys_id)
                 self.physical_memory.free(phys_id)
-                self.send_signal("memory free")  # TODO
+                self.send_signal(SIGNAL_MEMORY_FREED)
                 self.qdevice.mem_positions[phys_id].in_use = False
                 outcome = yield from self._measure_electron()
                 app_mem.set_reg_value(instr.creg, outcome)
@@ -803,7 +837,7 @@ class NVProcessor(Processor):
                 app_mem.map_virt_id(virt_id, 0)
                 yield from self._move_carbon_to_electron_for_measure(phys_id)
                 self.physical_memory.free(phys_id)
-                self.send_signal("memory free")  # TODO
+                self.send_signal(SIGNAL_MEMORY_FREED)
                 self.qdevice.mem_positions[phys_id].in_use = False
                 outcome = yield from self._measure_electron()
                 app_mem.set_reg_value(instr.creg, outcome)
